@@ -1,6 +1,4 @@
-from .handlers.base import ChannelsConsumer, ChannelsWSConsumer
-from .handlers.graphql_transport_ws_handler import GraphQLTransportWSHandler
-from .handlers.graphql_ws_handler import GraphQLWSHandler
+from .handlers.base import ChannelsConsumer
 from .handlers.http_handler import (
     ChannelsRequest,
     GraphQLHTTPConsumer,
@@ -12,11 +10,8 @@ from .router import GraphQLProtocolTypeRouter
 __all__ = [
     "ChannelsConsumer",
     "ChannelsRequest",
-    "ChannelsWSConsumer",
-    "GraphQLProtocolTypeRouter",
-    "GraphQLWSHandler",
-    "GraphQLTransportWSHandler",
     "GraphQLHTTPConsumer",
+    "GraphQLProtocolTypeRouter",
     "GraphQLWSConsumer",
     "SyncGraphQLHTTPConsumer",
 ]
