@@ -102,14 +102,13 @@ class GraphQLWSConsumer(
     ```
     """
 
-    websocket_adapter_class = ChannelsWebSocketAdapter
+    websocket_adapter_class = ChannelsWebSocketAdapter  # type: ignore
 
     def __init__(
         self,
         schema: BaseSchema,
         keep_alive: bool = False,
         keep_alive_interval: float = 1,
-        debug: bool = False,
         subscription_protocols: Sequence[str] = (
             GRAPHQL_TRANSPORT_WS_PROTOCOL,
             GRAPHQL_WS_PROTOCOL,
@@ -122,7 +121,6 @@ class GraphQLWSConsumer(
         self.schema = schema
         self.keep_alive = keep_alive
         self.keep_alive_interval = keep_alive_interval
-        self.debug = debug
         self.protocols = subscription_protocols
         self.message_queue: asyncio.Queue[MessageQueueData] = asyncio.Queue()
         self.run_task: Optional[asyncio.Task] = None
