@@ -8,9 +8,7 @@ from typing import (
     TYPE_CHECKING,
     Any,
     Callable,
-    List,
     Optional,
-    Set,
     Union,
     cast,
 )
@@ -59,12 +57,20 @@ try:
 except ImportError:
     TypeVarDef = TypeVarType
 
+PYDANTIC_VERSION: Optional[tuple[int, ...]] = None
+
 # To be compatible with user who don't use pydantic
 try:
+    import pydantic
     from pydantic.mypy import METADATA_KEY as PYDANTIC_METADATA_KEY
     from pydantic.mypy import PydanticModelField
+
+    PYDANTIC_VERSION = tuple(map(int, pydantic.__version__.split(".")))  # noqa: RUF048
+
+    from strawberry.experimental.pydantic._compat import IS_PYDANTIC_V1
 except ImportError:
     PYDANTIC_METADATA_KEY = ""
+    IS_PYDANTIC_V1 = False
 
 
 if TYPE_CHECKING:
@@ -83,7 +89,7 @@ FALLBACK_VERSION = Decimal("0.800")
 
 
 class MypyVersion:
-    """Stores the mypy version to be used by the plugin"""
+    """Stores the mypy version to be used by the plugin."""
 
     VERSION: Decimal
 
@@ -105,12 +111,10 @@ def lazy_type_analyze_callback(ctx: AnalyzeTypeContext) -> Type:
         return AnyType(TypeOfAny.special_form)
 
     type_name = ctx.type.args[0]
-    type_ = ctx.api.analyze_type(type_name)
-
-    return type_
+    return ctx.api.analyze_type(type_name)
 
 
-def _get_named_type(name: str, api: SemanticAnalyzerPluginInterface):
+def _get_named_type(name: str, api: SemanticAnalyzerPluginInterface) -> Any:
     if "." in name:
         return api.named_type_or_none(name)
 
@@ -141,14 +145,12 @@ def _get_type_for_expr(expr: Expression, api: SemanticAnalyzerPluginInterface) -
     if isinstance(expr, MemberExpr):
         if expr.fullname:
             return _get_named_type(expr.fullname, api)
-        else:
-            raise InvalidNodeTypeException(expr)
+        raise InvalidNodeTypeException(expr)
 
     if isinstance(expr, CallExpr):
         if expr.analyzed:
             return _get_type_for_expr(expr.analyzed, api)
-        else:
-            raise InvalidNodeTypeException(expr)
+        raise InvalidNodeTypeException(expr)
 
     if isinstance(expr, CastExpr):
         return expr.type
@@ -172,8 +174,6 @@ def create_type_hook(ctx: DynamicClassDefContext) -> None:
         SymbolTableNode(GDEF, type_alias, plugin_generated=True),
     )
 
-    return
-
 
 def union_hook(ctx: DynamicClassDefContext) -> None:
     try:
@@ -317,13 +317,15 @@ def add_static_method_to_class(
     api: Union[SemanticAnalyzerPluginInterface, CheckerPluginInterface],
     cls: ClassDef,
     name: str,
-    args: List[Argument],
+    args: list[Argument],
     return_type: Type,
     tvar_def: Optional[TypeVarType] = None,
 ) -> None:
-    """Adds a static method
-    Edited add_method_to_class to incorporate static method logic
-    https://github.com/python/mypy/blob/9c05d3d19/mypy/plugins/common.py
+    """Adds a static method.
+
+    Edited `add_method_to_class` to incorporate static method logic
+
+    https://github.com/python/mypy/blob/9c05d3d19/mypy/plugins/common.py.
     """
     info = cls.info
 
@@ -335,13 +337,12 @@ def add_static_method_to_class(
             cls.defs.body.remove(sym.node)
 
     # For compat with mypy < 0.93
-    if MypyVersion.VERSION < Decimal("0.93"):
+    if Decimal("0.93") > MypyVersion.VERSION:
         function_type = api.named_type("__builtins__.function")
+    elif isinstance(api, SemanticAnalyzerPluginInterface):
+        function_type = api.named_type("builtins.function")
     else:
-        if isinstance(api, SemanticAnalyzerPluginInterface):
-            function_type = api.named_type("builtins.function")
-        else:
-            function_type = api.named_generic_type("builtins.function", [])
+        function_type = api.named_generic_type("builtins.function", [])
 
     arg_types, arg_names, arg_kinds = [], [], []
     for arg in args:
@@ -354,7 +355,7 @@ def add_static_method_to_class(
         arg_types, arg_kinds, arg_names, return_type, function_type
     )
     if tvar_def:
-        signature.variables = [tvar_def]
+        signature.variables = [tvar_def]  # type: ignore[assignment]
 
     func = FuncDef(name, args, Block([PassStmt()]))
 
@@ -396,23 +397,27 @@ def strawberry_pydantic_class_callback(ctx: ClassDefContext) -> None:
         ]
         add_method(ctx, "__init__", init_args, NoneType())
 
-        model_type = cast(Instance, _get_type_for_expr(model_expression, ctx.api))
+        model_type = cast("Instance", _get_type_for_expr(model_expression, ctx.api))
 
         # these are the fields that the user added to the strawberry type
-        new_strawberry_fields: Set[str] = set()
+        new_strawberry_fields: set[str] = set()
 
         # TODO: think about inheritance for strawberry?
         for stmt in ctx.cls.defs.body:
             if isinstance(stmt, AssignmentStmt):
-                lhs = cast(NameExpr, stmt.lvalues[0])
+                lhs = cast("NameExpr", stmt.lvalues[0])
                 new_strawberry_fields.add(lhs.name)
 
-        pydantic_fields: Set[PydanticModelField] = set()
+        pydantic_fields: set[PydanticModelField] = set()
         try:
-            for _name, data in model_type.type.metadata[PYDANTIC_METADATA_KEY][
-                "fields"
-            ].items():
-                field = PydanticModelField.deserialize(ctx.cls.info, data)
+            fields = model_type.type.metadata[PYDANTIC_METADATA_KEY]["fields"]
+            for data in fields.items():
+                if IS_PYDANTIC_V1:
+                    field = PydanticModelField.deserialize(ctx.cls.info, data[1])  # type:ignore[call-arg]
+                else:
+                    field = PydanticModelField.deserialize(
+                        info=ctx.cls.info, data=data[1], api=ctx.api
+                    )
                 pydantic_fields.add(field)
         except KeyError:
             # this will happen if the user didn't add the pydantic plugin
@@ -423,7 +428,7 @@ def strawberry_pydantic_class_callback(ctx: ClassDefContext) -> None:
                 ctx.reason,
             )
 
-        potentially_missing_fields: Set[PydanticModelField] = {
+        potentially_missing_fields: set[PydanticModelField] = {
             f for f in pydantic_fields if f.name not in new_strawberry_fields
         }
 
@@ -434,27 +439,62 @@ def strawberry_pydantic_class_callback(ctx: ClassDefContext) -> None:
         This means that the user is using all_fields=True
         """
         is_all_fields: bool = len(potentially_missing_fields) == len(pydantic_fields)
-        missing_pydantic_fields: Set[PydanticModelField] = (
+        missing_pydantic_fields: set[PydanticModelField] = (
             potentially_missing_fields if not is_all_fields else set()
         )
 
         # Add the default to_pydantic if undefined by the user
         if "to_pydantic" not in ctx.cls.info.names:
-            add_method(
-                ctx,
-                "to_pydantic",
-                args=[
-                    f.to_argument(
-                        # TODO: use_alias should depend on config?
-                        info=model_type.type,
-                        typed=True,
-                        force_optional=False,
-                        use_alias=True,
-                    )
-                    for f in missing_pydantic_fields
-                ],
-                return_type=model_type,
-            )
+            if IS_PYDANTIC_V1:
+                add_method(
+                    ctx,
+                    "to_pydantic",
+                    args=[
+                        f.to_argument(
+                            # TODO: use_alias should depend on config?
+                            info=model_type.type,  # type:ignore[call-arg]
+                            typed=True,
+                            force_optional=False,
+                            use_alias=True,
+                        )
+                        for f in missing_pydantic_fields
+                    ],
+                    return_type=model_type,
+                )
+            else:
+                extra = {}
+
+                if PYDANTIC_VERSION:
+                    if PYDANTIC_VERSION >= (2, 7, 0):
+                        extra["api"] = ctx.api
+                    if PYDANTIC_VERSION >= (2, 8, 0):
+                        # Based on pydantic's default value
+                        # https://github.com/pydantic/pydantic/pull/9606/files#diff-469037bbe55bbf9aa359480a16040d368c676adad736e133fb07e5e20d6ac523R1066
+                        extra["force_typevars_invariant"] = False
+                    if PYDANTIC_VERSION >= (2, 9, 0):
+                        extra["model_strict"] = model_type.type.metadata[
+                            PYDANTIC_METADATA_KEY
+                        ]["config"].get("strict", False)
+                        extra["is_root_model_root"] = any(
+                            "pydantic.root_model.RootModel" in base.fullname
+                            for base in model_type.type.mro[:-1]
+                        )
+                add_method(
+                    ctx,
+                    "to_pydantic",
+                    args=[
+                        f.to_argument(
+                            # TODO: use_alias should depend on config?
+                            current_info=model_type.type,
+                            typed=True,
+                            force_optional=False,
+                            use_alias=True,
+                            **extra,
+                        )
+                        for f in missing_pydantic_fields
+                    ],
+                    return_type=model_type,
+                )
 
         # Add from_pydantic
         model_argument = Argument(
@@ -463,12 +503,23 @@ def strawberry_pydantic_class_callback(ctx: ClassDefContext) -> None:
             initializer=None,
             kind=ARG_OPT,
         )
+        extra_type = ctx.api.named_type(
+            "builtins.dict",
+            [ctx.api.named_type("builtins.str"), AnyType(TypeOfAny.explicit)],
+        )
+
+        extra_argument = Argument(
+            variable=Var(name="extra", type=UnionType([NoneType(), extra_type])),
+            type_annotation=UnionType([NoneType(), extra_type]),
+            initializer=None,
+            kind=ARG_OPT,
+        )
 
         add_static_method_to_class(
             ctx.api,
             ctx.cls,
             name="from_pydantic",
-            args=[model_argument],
+            args=[model_argument, extra_argument],
             return_type=fill_typevars(ctx.cls.info),
         )
 
@@ -508,22 +559,22 @@ class StrawberryPlugin(Plugin):
         return None
 
     def _is_strawberry_union(self, fullname: str) -> bool:
-        return fullname == "strawberry.union.union" or fullname.endswith(
+        return fullname == "strawberry.types.union.union" or fullname.endswith(
             "strawberry.union"
         )
 
     def _is_strawberry_enum(self, fullname: str) -> bool:
-        return fullname == "strawberry.enum.enum" or fullname.endswith(
+        return fullname == "strawberry.types.enum.enum" or fullname.endswith(
             "strawberry.enum"
         )
 
     def _is_strawberry_scalar(self, fullname: str) -> bool:
-        return fullname == "strawberry.custom_scalar.scalar" or fullname.endswith(
+        return fullname == "strawberry.types.scalar.scalar" or fullname.endswith(
             "strawberry.scalar"
         )
 
     def _is_strawberry_lazy_type(self, fullname: str) -> bool:
-        return fullname == "strawberry.lazy_type.LazyType"
+        return fullname == "strawberry.types.lazy_type.LazyType"
 
     def _is_strawberry_create_type(self, fullname: str) -> bool:
         # using endswith(.create_type) is not ideal as there might be
