@@ -4,54 +4,57 @@ import dataclasses
 import inspect
 import itertools
 import sys
-from typing import (
-    TYPE_CHECKING,
-    Any,
+from collections.abc import (
     AsyncIterable,
     AsyncIterator,
     Awaitable,
-    ClassVar,
-    Generic,
     Iterable,
     Iterator,
-    List,
-    Optional,
     Sequence,
-    Type,
+)
+from typing import (
+    TYPE_CHECKING,
+    Annotated,
+    Any,
+    ClassVar,
+    ForwardRef,
+    Generic,
+    Optional,
     TypeVar,
     Union,
     cast,
     overload,
 )
-from typing_extensions import (
-    Annotated,
-    Literal,
-    Self,
-    TypeAlias,
-    get_args,
-    get_origin,
-)
+from typing_extensions import Literal, Self, TypeAlias, get_args, get_origin
 
-from strawberry.field import field
-from strawberry.lazy_type import LazyType
-from strawberry.object_type import interface, type
-from strawberry.private import StrawberryPrivate
 from strawberry.relay.exceptions import NodeIDAnnotationError
-from strawberry.type import StrawberryContainer
-from strawberry.types.types import TypeDefinition
-from strawberry.utils.aio import aenumerate, aislice, resolve_awaitable
+from strawberry.types.base import (
+    StrawberryContainer,
+    StrawberryObjectDefinition,
+    get_object_definition,
+)
+from strawberry.types.field import field
+from strawberry.types.info import Info  # noqa: TC001
+from strawberry.types.lazy_type import LazyType
+from strawberry.types.object_type import interface
+from strawberry.types.object_type import type as strawberry_type
+from strawberry.types.private import StrawberryPrivate
+from strawberry.utils.aio import aclosing, aenumerate, aislice, resolve_awaitable
 from strawberry.utils.inspect import in_async_context
-from strawberry.utils.typing import eval_type
+from strawberry.utils.typing import eval_type, is_classvar
 
-from .utils import from_base64, to_base64
+from .utils import (
+    SliceMetadata,
+    from_base64,
+    should_resolve_list_connection_edges,
+    to_base64,
+)
 
 if TYPE_CHECKING:
     from strawberry.scalars import ID
-    from strawberry.types.info import Info
     from strawberry.utils.await_maybe import AwaitableOrValue
 
 _T = TypeVar("_T")
-_R = TypeVar("_R")
 
 NodeIterableType: TypeAlias = Union[
     Iterator[_T],
@@ -93,7 +96,7 @@ class GlobalID:
     type_name: str
     node_id: str
 
-    def __post_init__(self):
+    def __post_init__(self) -> None:
         if not isinstance(self.type_name, str):
             raise GlobalIDValueError(
                 f"type_name is expected to be a string, found {self.type_name!r}"
@@ -103,7 +106,7 @@ class GlobalID:
                 f"node_id is expected to be a string, found {self.node_id!r}"
             )
 
-    def __str__(self):
+    def __str__(self) -> str:
         return to_base64(self.type_name, self.node_id)
 
     @classmethod
@@ -136,9 +139,8 @@ class GlobalID:
         info: Info,
         *,
         required: Literal[True] = ...,
-        ensure_type: Type[_T],
-    ) -> _T:
-        ...
+        ensure_type: type[_T],
+    ) -> _T: ...
 
     @overload
     async def resolve_node(
@@ -147,8 +149,7 @@ class GlobalID:
         *,
         required: Literal[True],
         ensure_type: None = ...,
-    ) -> Node:
-        ...
+    ) -> Node: ...
 
     @overload
     async def resolve_node(
@@ -157,8 +158,7 @@ class GlobalID:
         *,
         required: bool = ...,
         ensure_type: None = ...,
-    ) -> Optional[Node]:
-        ...
+    ) -> Optional[Node]: ...
 
     async def resolve_node(self, info, *, required=False, ensure_type=None) -> Any:
         """Resolve the type name and node id info to the node itself.
@@ -187,8 +187,8 @@ class GlobalID:
 
         """
         n_type = self.resolve_type(info)
-        node = cast(
-            Awaitable[Node],
+        node: Node | Awaitable[Node] = cast(
+            "Awaitable[Node]",
             n_type.resolve_node(
                 self.node_id,
                 info=info,
@@ -205,11 +205,15 @@ class GlobalID:
                 ensure_type = tuple(get_args(ensure_type))
 
             if not isinstance(node, ensure_type):
-                raise TypeError(f"{ensure_type} expected, found {node!r}")
+                msg = (
+                    f"Cannot resolve. GlobalID requires {ensure_type}, received {node!r}. "
+                    "Verify that the supplied ID is intended for this Query/Mutation/Subscription."
+                )
+                raise TypeError(msg)
 
         return node
 
-    def resolve_type(self, info: Info) -> Type[Node]:
+    def resolve_type(self, info: Info) -> type[Node]:
         """Resolve the internal type name to its type itself.
 
         Args:
@@ -220,16 +224,23 @@ class GlobalID:
             The resolved GraphQL type for the execution info
 
         """
-        schema = info.schema
         type_def = info.schema.get_type_by_name(self.type_name)
-        assert isinstance(type_def, TypeDefinition)
+        if not isinstance(type_def, StrawberryObjectDefinition):
+            raise GlobalIDValueError(
+                f"Cannot resolve. GlobalID requires a GraphQL type, "
+                f"received `{self.type_name}`."
+            )
 
         origin = (
             type_def.origin.resolve_type
             if isinstance(type_def.origin, LazyType)
             else type_def.origin
         )
-        assert issubclass(origin, Node)
+        if not issubclass(origin, Node):
+            raise GlobalIDValueError(
+                f"Cannot resolve. GlobalID requires a GraphQL Node type, "
+                f"received `{self.type_name}`."
+            )
         return origin
 
     @overload
@@ -238,9 +249,8 @@ class GlobalID:
         info: Info,
         *,
         required: Literal[True] = ...,
-        ensure_type: Type[_T],
-    ) -> _T:
-        ...
+        ensure_type: type[_T],
+    ) -> _T: ...
 
     @overload
     def resolve_node_sync(
@@ -249,8 +259,7 @@ class GlobalID:
         *,
         required: Literal[True],
         ensure_type: None = ...,
-    ) -> Node:
-        ...
+    ) -> Node: ...
 
     @overload
     def resolve_node_sync(
@@ -259,8 +268,7 @@ class GlobalID:
         *,
         required: bool = ...,
         ensure_type: None = ...,
-    ) -> Optional[Node]:
-        ...
+    ) -> Optional[Node]: ...
 
     def resolve_node_sync(self, info, *, required=False, ensure_type=None) -> Any:
         """Resolve the type name and node id info to the node itself.
@@ -301,7 +309,11 @@ class GlobalID:
                 ensure_type = tuple(get_args(ensure_type))
 
             if not isinstance(node, ensure_type):
-                raise TypeError(f"{ensure_type} expected, found {node!r}")
+                msg = (
+                    f"Cannot resolve. GlobalID requires {ensure_type}, received {node!r}. "
+                    "Verify that the supplied ID is intended for this Query/Mutation/Subscription."
+                )
+                raise TypeError(msg)
 
         return node
 
@@ -312,9 +324,14 @@ class NodeIDPrivate(StrawberryPrivate):
     The `Node` interface will automatically create and resolve GlobalIDs
     based on the field annotated with `NodeID`. e.g:
 
-      >>> @strawberry.type
-      ... class Fruit(Node):
-      ...     code: NodeID[str]
+    ```python
+    import strawberry
+
+
+    @strawberry.type
+    class Fruit(Node):
+        code: NodeID[str]
+    ```
 
     In this case, `code` will be used to generate a global ID in the
     format `Fruit:<code>` and will be exposed as `id: GlobalID!` in the
@@ -345,17 +362,20 @@ class Node:
             single node id.
 
     Example:
+    ```python
+    import strawberry
 
-        >>> @strawberry.type
-        ... class Fruit(Node):
-        ...     id: NodeID[int]
-        ...     name: str
-        ...
-        ... @classmethod
-        ... def resolve_nodes(cls, *, info, node_ids, required=False):
-        ...     # Return an iterable of fruits in here
-        ...     ...
 
+    @strawberry.type
+    class Fruit(strawberry.relay.Node):
+        id: strawberry.relay.NodeID[int]
+        name: str
+
+        @classmethod
+        def resolve_nodes(cls, *, info, node_ids, required=False):
+            # Return an iterable of fruits in here
+            ...
+    ```
     """
 
     _id_attr: ClassVar[Optional[str]] = None
@@ -372,8 +392,8 @@ class Node:
         else:
             parent_type = info._raw_info.parent_type
             type_def = info.schema.get_type_by_name(parent_type.name)
-            assert isinstance(type_def, TypeDefinition)
-            origin = cast(Type[Node], type_def.origin)
+            assert isinstance(type_def, StrawberryObjectDefinition)
+            origin = cast("type[Node]", type_def.origin)
             resolve_id = origin.resolve_id
             resolve_typename = origin.resolve_typename
 
@@ -384,7 +404,7 @@ class Node:
 
         if inspect.isawaitable(node_id):
             return cast(
-                GlobalID,
+                "GlobalID",
                 resolve_awaitable(
                     node_id,
                     lambda resolved: GlobalID(
@@ -407,7 +427,16 @@ class Node:
             base_namespace = sys.modules[base.__module__].__dict__
 
             for attr_name, attr in getattr(base, "__annotations__", {}).items():
-                evaled = eval_type(attr, globalns=base_namespace)
+                # Some ClassVar might raise TypeError when being resolved
+                # on some python versions. This is fine to skip since
+                # we are not interested in ClassVars here
+                if is_classvar(base, attr):
+                    continue
+
+                evaled = eval_type(
+                    ForwardRef(attr) if isinstance(attr, str) else attr,
+                    globalns=base_namespace,
+                )
 
                 if get_origin(evaled) is Annotated and any(
                     isinstance(a, NodeIDPrivate) for a in get_args(evaled)
@@ -451,10 +480,8 @@ class Node:
         You can override this method to provide a custom implementation.
 
         Args:
-            info:
-                The strawberry execution info resolve the type name from
-            root:
-                The node to resolve
+            info: The strawberry execution info resolve the type name from.
+            root: The node to resolve.
 
         Returns:
             The resolved id (which is expected to be str)
@@ -476,8 +503,7 @@ class Node:
         info: Info,
         node_ids: Iterable[str],
         required: Literal[True],
-    ) -> AwaitableOrValue[Iterable[Self]]:
-        ...
+    ) -> AwaitableOrValue[Iterable[Self]]: ...
 
     @overload
     @classmethod
@@ -487,8 +513,7 @@ class Node:
         info: Info,
         node_ids: Iterable[str],
         required: Literal[False] = ...,
-    ) -> AwaitableOrValue[Iterable[Optional[Self]]]:
-        ...
+    ) -> AwaitableOrValue[Iterable[Optional[Self]]]: ...
 
     @overload
     @classmethod
@@ -501,8 +526,7 @@ class Node:
     ) -> Union[
         AwaitableOrValue[Iterable[Self]],
         AwaitableOrValue[Iterable[Optional[Self]]],
-    ]:
-        ...
+    ]: ...
 
     @classmethod
     def resolve_nodes(
@@ -522,12 +546,9 @@ class Node:
         returned as `None`.
 
         Args:
-            info:
-                The strawberry execution info resolve the type name from
-            node_ids:
-                List of node ids that should be returned
-            required:
-                If `True`, all `node_ids` requested must exist. If they don't,
+            info: The strawberry execution info resolve the type name from.
+            node_ids: List of node ids that should be returned.
+            required: If `True`, all `node_ids` requested must exist. If they don't,
                 an error must be raised. If `False`, missing nodes should be
                 returned as `None`. It only makes sense when passing a list of
                 `node_ids`, otherwise it will should ignored.
@@ -546,8 +567,7 @@ class Node:
         *,
         info: Info,
         required: Literal[True],
-    ) -> AwaitableOrValue[Self]:
-        ...
+    ) -> AwaitableOrValue[Self]: ...
 
     @overload
     @classmethod
@@ -557,8 +577,7 @@ class Node:
         *,
         info: Info,
         required: Literal[False] = ...,
-    ) -> AwaitableOrValue[Optional[Self]]:
-        ...
+    ) -> AwaitableOrValue[Optional[Self]]: ...
 
     @overload
     @classmethod
@@ -568,8 +587,7 @@ class Node:
         *,
         info: Info,
         required: bool,
-    ) -> AwaitableOrValue[Optional[Self]]:
-        ...
+    ) -> AwaitableOrValue[Optional[Self]]: ...
 
     @classmethod
     def resolve_node(
@@ -585,28 +603,24 @@ class Node:
         a single node id.
 
         Args:
-            info:
-                The strawberry execution info resolve the type name from
-            node_id:
-                The id of the node to be retrieved
-            required:
-                if the node is required or not to exist. If not, then None
+            info: The strawberry execution info resolve the type name from.
+            node_id: The id of the node to be retrieved.
+            required: if the node is required or not to exist. If not, then None
                 should be returned if it doesn't exist. Otherwise an exception
                 should be raised.
 
         Returns:
             The resolved node or None if it was not found
-
         """
         retval = cls.resolve_nodes(info=info, node_ids=[node_id], required=required)
 
         if inspect.isawaitable(retval):
             return resolve_awaitable(retval, lambda resolved: next(iter(resolved)))
 
-        return next(iter(cast(Iterable[Self], retval)))
+        return next(iter(cast("Iterable[Self]", retval)))
 
 
-@type(description="Information to aid in pagination.")
+@strawberry_type(description="Information to aid in pagination.")
 class PageInfo:
     """Information to aid in pagination.
 
@@ -619,7 +633,6 @@ class PageInfo:
             When paginating backwards, the cursor to continue
         end_cursor:
             When paginating forwards, the cursor to continue
-
     """
 
     has_next_page: bool = field(
@@ -636,7 +649,7 @@ class PageInfo:
     )
 
 
-@type(description="An edge in a connection.")
+@strawberry_type(description="An edge in a connection.")
 class Edge(Generic[NodeType]):
     """An edge in a connection.
 
@@ -645,22 +658,19 @@ class Edge(Generic[NodeType]):
             A cursor for use in pagination
         node:
             The item at the end of the edge
-
     """
 
-    cursor: str = field(
-        description="A cursor for use in pagination",
-    )
-    node: NodeType = field(
-        description="The item at the end of the edge",
-    )
+    cursor: str = field(description="A cursor for use in pagination")
+    node: NodeType = field(description="The item at the end of the edge")
+
+    CURSOR_PREFIX: ClassVar[str] = PREFIX
 
     @classmethod
-    def resolve_edge(cls, node: NodeType, *, cursor: Any = None) -> Self:
-        return cls(cursor=to_base64(PREFIX, cursor), node=node)
+    def resolve_edge(cls, node: NodeType, *, cursor: Any = None, **kwargs: Any) -> Self:
+        return cls(cursor=to_base64(cls.CURSOR_PREFIX, cursor), node=node, **kwargs)
 
 
-@type(description="A connection to a list of items.")
+@strawberry_type(description="A connection to a list of items.")
 class Connection(Generic[NodeType]):
     """A connection to a list of items.
 
@@ -672,11 +682,9 @@ class Connection(Generic[NodeType]):
 
     """
 
-    page_info: PageInfo = field(
-        description="Pagination data for this connection",
-    )
-    edges: List[Edge[NodeType]] = field(
-        description="Contains the nodes in this connection",
+    page_info: PageInfo = field(description="Pagination data for this connection")
+    edges: list[Edge[NodeType]] = field(
+        description="Contains the nodes in this connection"
     )
 
     @classmethod
@@ -692,9 +700,11 @@ class Connection(Generic[NodeType]):
         Args:
             node:
                 The resolved node which should return an instance of this
-                connection's `NodeType`
+                connection's `NodeType`.
             info:
-                The strawberry execution info resolve the type name from
+                The strawberry execution info resolve the type name from.
+            **kwargs:
+                Additional arguments passed to the resolver.
 
         """
         return node
@@ -709,6 +719,7 @@ class Connection(Generic[NodeType]):
         after: Optional[str] = None,
         first: Optional[int] = None,
         last: Optional[int] = None,
+        max_results: Optional[int] = None,
         **kwargs: Any,
     ) -> AwaitableOrValue[Self]:
         """Resolve a connection from nodes.
@@ -717,18 +728,14 @@ class Connection(Generic[NodeType]):
         on `first`/`last`/`before`/`after` arguments.
 
         Args:
-            info:
-                The strawberry execution info resolve the type name from
-            nodes:
-                An iterable/iteretor of nodes to paginate
-            before:
-                Returns the items in the list that come before the specified cursor
-            after:
-                Returns the items in the list that come after the specified cursor
-            first:
-                Returns the first n items from the list
-            last:
-                Returns the items in the list that come after the specified cursor
+            info: The strawberry execution info resolve the type name from.
+            nodes: An iterable/iteretor of nodes to paginate.
+            before: Returns the items in the list that come before the specified cursor.
+            after: Returns the items in the list that come after the specified cursor.
+            first: Returns the first n items from the list.
+            last: Returns the items in the list that come after the specified cursor.
+            max_results: The maximum number of results to resolve.
+            kwargs: Additional arguments passed to the resolver.
 
         Returns:
             The resolved `Connection`
@@ -737,7 +744,7 @@ class Connection(Generic[NodeType]):
         raise NotImplementedError
 
 
-@type(name="Connection", description="A connection to a list of items.")
+@strawberry_type(name="Connection", description="A connection to a list of items.")
 class ListConnection(Connection[NodeType]):
     """A connection to a list of items.
 
@@ -749,11 +756,9 @@ class ListConnection(Connection[NodeType]):
 
     """
 
-    page_info: PageInfo = field(
-        description="Pagination data for this connection",
-    )
-    edges: List[Edge[NodeType]] = field(
-        description="Contains the nodes in this connection",
+    page_info: PageInfo = field(description="Pagination data for this connection")
+    edges: list[Edge[NodeType]] = field(
+        description="Contains the nodes in this connection"
     )
 
     @classmethod
@@ -766,6 +771,7 @@ class ListConnection(Connection[NodeType]):
         after: Optional[str] = None,
         first: Optional[int] = None,
         last: Optional[int] = None,
+        max_results: Optional[int] = None,
         **kwargs: Any,
     ) -> AwaitableOrValue[Self]:
         """Resolve a connection from the list of nodes.
@@ -773,115 +779,90 @@ class ListConnection(Connection[NodeType]):
         This uses the described Relay Pagination algorithm_
 
         Args:
-            info:
-                The strawberry execution info resolve the type name from
-            nodes:
-                An iterable/iteretor of nodes to paginate
-            before:
-                Returns the items in the list that come before the specified cursor
-            after:
-                Returns the items in the list that come after the specified cursor
-            first:
-                Returns the first n items from the list
-            last:
-                Returns the items in the list that come after the specified cursor
+            info: The strawberry execution info resolve the type name from.
+            nodes: An iterable/iteretor of nodes to paginate.
+            before: Returns the items in the list that come before the specified cursor.
+            after: Returns the items in the list that come after the specified cursor.
+            first: Returns the first n items from the list.
+            last: Returns the items in the list that come after the specified cursor.
+            max_results: The maximum number of results to resolve.
+            kwargs: Additional arguments passed to the resolver.
 
         Returns:
             The resolved `Connection`
 
         .. _Relay Pagination algorithm:
             https://relay.dev/graphql/connections.htm#sec-Pagination-algorithm
-
         """
-        max_results = info.schema.config.relay_max_results
-        start = 0
-        end: Optional[int] = None
-
-        if after:
-            after_type, after_parsed = from_base64(after)
-            assert after_type == PREFIX
-            start = int(after_parsed) + 1
-        if before:
-            before_type, before_parsed = from_base64(before)
-            assert before_type == PREFIX
-            end = int(before_parsed)
-
-        if isinstance(first, int):
-            if first < 0:
-                raise ValueError("Argument 'first' must be a non-negative integer.")
-
-            if first > max_results:
-                raise ValueError(
-                    f"Argument 'first' cannot be higher than {max_results}."
-                )
-
-            if end is not None:
-                start = max(0, end - 1)
-
-            end = start + first
-        if isinstance(last, int):
-            if last < 0:
-                raise ValueError("Argument 'last' must be a non-negative integer.")
-
-            if last > max_results:
-                raise ValueError(
-                    f"Argument 'last' cannot be higher than {max_results}."
-                )
-
-            if end is not None:
-                start = max(start, end - last)
-            else:
-                end = sys.maxsize
-
-        if end is None:
-            end = max_results
-
-        expected = end - start if end != sys.maxsize else None
-        # Overfetch by 1 to check if we have a next result
-        overfetch = end + 1 if end != sys.maxsize else end
-
-        type_def = cast(TypeDefinition, cls._type_definition)  # type:ignore
+        type_def = get_object_definition(cls)
+        assert type_def
         field_def = type_def.get_field("edges")
         assert field_def
 
-        field = field_def.type
+        field = field_def.resolve_type(type_definition=type_def)
         while isinstance(field, StrawberryContainer):
             field = field.of_type
 
-        edge_class = cast(Edge[NodeType], field)
+        edge_class = cast("Edge[NodeType]", field)
+
+        slice_metadata = SliceMetadata.from_arguments(
+            info,
+            before=before,
+            after=after,
+            first=first,
+            last=last,
+            max_results=max_results,
+            prefix=edge_class.CURSOR_PREFIX,
+        )
 
         if isinstance(nodes, (AsyncIterator, AsyncIterable)) and in_async_context():
 
-            async def resolver():
+            async def resolver() -> Self:
                 try:
                     iterator = cast(
-                        Union[AsyncIterator[NodeType], AsyncIterable[NodeType]],
-                        cast(Sequence, nodes)[start:overfetch],
+                        "Union[AsyncIterator[NodeType], AsyncIterable[NodeType]]",
+                        cast("Sequence", nodes)[
+                            slice_metadata.start : slice_metadata.overfetch
+                        ],
                     )
                 except TypeError:
                     # TODO: Why mypy isn't narrowing this based on the if above?
                     assert isinstance(nodes, (AsyncIterator, AsyncIterable))
                     iterator = aislice(
                         nodes,
-                        start,
-                        overfetch,
+                        slice_metadata.start,
+                        slice_metadata.overfetch,
                     )
 
-                assert isinstance(iterator, (AsyncIterator, AsyncIterable))
-                edges: List[Edge] = [
-                    edge_class.resolve_edge(
-                        cls.resolve_node(v, info=info, **kwargs),
-                        cursor=start + i,
-                    )
-                    async for i, v in aenumerate(iterator)
-                ]
+                async with aclosing(iterator):
+                    # The slice above might return an object that now is not async
+                    # iterable anymore (e.g. an already cached django queryset)
+                    if isinstance(iterator, (AsyncIterator, AsyncIterable)):
+                        edges: list[Edge] = [
+                            edge_class.resolve_edge(
+                                cls.resolve_node(v, info=info, **kwargs),
+                                cursor=slice_metadata.start + i,
+                            )
+                            async for i, v in aenumerate(iterator)
+                        ]
+                    else:
+                        edges: list[Edge] = [  # type: ignore[no-redef]
+                            edge_class.resolve_edge(
+                                cls.resolve_node(v, info=info, **kwargs),
+                                cursor=slice_metadata.start + i,
+                            )
+                            for i, v in enumerate(iterator)
+                        ]
 
-                has_previous_page = start > 0
-                if expected is not None and len(edges) == expected + 1:
+                has_previous_page = slice_metadata.start > 0
+                if (
+                    slice_metadata.expected is not None
+                    and len(edges) == slice_metadata.expected + 1
+                ):
                     # Remove the overfetched result
                     edges = edges[:-1]
                     has_next_page = True
-                elif end == sys.maxsize:
+                elif slice_metadata.end == sys.maxsize:
                     # Last was asked without any after/before
                     assert last is not None
                     original_len = len(edges)
@@ -905,31 +886,47 @@ class ListConnection(Connection[NodeType]):
 
         try:
             iterator = cast(
-                Union[Iterator[NodeType], Iterable[NodeType]],
-                cast(Sequence, nodes)[start:overfetch],
+                "Union[Iterator[NodeType], Iterable[NodeType]]",
+                cast("Sequence", nodes)[
+                    slice_metadata.start : slice_metadata.overfetch
+                ],
             )
         except TypeError:
             assert isinstance(nodes, (Iterable, Iterator))
             iterator = itertools.islice(
                 nodes,
-                start,
-                overfetch,
+                slice_metadata.start,
+                slice_metadata.overfetch,
+            )
+
+        if not should_resolve_list_connection_edges(info):
+            return cls(
+                edges=[],
+                page_info=PageInfo(
+                    start_cursor=None,
+                    end_cursor=None,
+                    has_previous_page=False,
+                    has_next_page=False,
+                ),
             )
 
         edges = [
             edge_class.resolve_edge(
                 cls.resolve_node(v, info=info, **kwargs),
-                cursor=start + i,
+                cursor=slice_metadata.start + i,
             )
             for i, v in enumerate(iterator)
         ]
 
-        has_previous_page = start > 0
-        if expected is not None and len(edges) == expected + 1:
+        has_previous_page = slice_metadata.start > 0
+        if (
+            slice_metadata.expected is not None
+            and len(edges) == slice_metadata.expected + 1
+        ):
             # Remove the overfetched result
             edges = edges[:-1]
             has_next_page = True
-        elif end == sys.maxsize:
+        elif slice_metadata.end == sys.maxsize:
             # Last was asked without any after/before
             assert last is not None
             original_len = len(edges)
@@ -948,3 +945,20 @@ class ListConnection(Connection[NodeType]):
                 has_next_page=has_next_page,
             ),
         )
+
+
+__all__ = [
+    "PREFIX",
+    "Connection",
+    "Edge",
+    "GlobalID",
+    "GlobalIDValueError",
+    "ListConnection",
+    "Node",
+    "NodeID",
+    "NodeIDAnnotationError",
+    "NodeIDPrivate",
+    "NodeIterableType",
+    "NodeType",
+    "PageInfo",
+]
